//============================================================================
// Название    : Task6.cpp
// Автор       : Сергей Синица
// Компиляция  : g++ Task6.cpp
//============================================================================

#include <iostream>
#include <fstream>
#include <vector>

#define DIG_RADIUS 0.5

using namespace std;

bool circle_cross_segment(double x1, double y1, double x2, double y2,
                          double xc, double yc, double r) {
  // сдвигаем чтобы окружность была в центре координат
  x1 -= xc;
  y1 -= yc;
  x2 -= xc;
  y2 -= yc;

  double dx = x2 - x1;
  double dy = y2 - y1;

  // составляем коэффициенты квадратного уравнения на пересечение прямой и окружности
  // если на отрезке [0..1] есть отрицательные значения, значит отрезок пересекает окружность
  double a = dx * dx + dy * dy;
  double b = 2. * (x1 * dx + y1 * dy);
  double c = x1 * x1 + y1 * y1 - r * r;

  // проверяем, есть ли на отрезке [0..1] решения
  if (-b < 0)
    return (c < 0);
  if (-b < (2. * a))
    return ((4. * a * c - b * b) < 0);

  return (a + b + c < 0);
}

bool point_in_polygon(int x, int y, std::vector<std::vector<double> > & area) {
    int i, j;
    bool c = false;

    for ( i = 0, j = area.size() - 1; i < area.size(); j = i++)
	{
        if ((((area[i][1] <= y) && (y < area[j][1])) || ((area[j][1] <= y) && (y < area[i][1]))) &&
        		(x < (area[j][0] - area[i][0]) * (y - area[i][1]) / (area[j][1] - area[i][1]) + area[i][0]))
            c = !c;
    }
	return c;
}

bool circle_in_polygon(std::vector<std::vector<double> > & area,
                          double xc, double yc, double r) {
	for (int i = 0; i < area.size(); i++) {
		int x1 = area[i][0];
		int y1 = area[i][1];
		int j = i +1;
		if (j == area.size()) {
			j = 0;
		}
		int x2 = area[j][0];
		int y2 = area[j][1];
		if (circle_cross_segment(x1, y1, x2, y2, xc, yc, r)) {
			return false;
		}
	}
	return point_in_polygon(xc, yc, area);
}

bool point_in_circle(double x, double y, double xc, double yc, double r) {
	return r * r > (x - xc) * (x - xc) + (y - yc) * (y - yc);
}

// проверяем сочетание и ищем максимум
void check_combination(std::vector<std::vector<double> > & points_can_dig,
		std::vector<std::vector<double> > & gold_can_dig,
		std::vector<int> & combination,
		int max_dig_depth,
		int & max_gold) {

	// проверяем, что мы не копаем в одном месте два раза, выбираем максимум
	std::vector<std::vector<double> > dig;
	for (unsigned int i = 0; i < combination.size(); i++) {
		int point_number = combination[i] - 1;
		std::vector<double> point_dig =  points_can_dig[point_number];
		double gx = point_dig[0];
		double gy = point_dig[1];
		double gz = point_dig[2];
		bool digged = false;
		for (unsigned int j = 0; j < dig.size(); j++) {
			if ((dig[j][0] == gx) && (dig[j][1] == gy) && (dig[j][2] < gz)) {
				dig[j][2] = gz;
				digged = true;
				break;
			}
		}
		if (!digged) {
			dig.push_back(point_dig);
		}
	}

	// проверяем не привышена ли суммарное время на раскопки
	int dig_depth = 0;
	for (unsigned int i = 0; i < dig.size(); i++) {
		std::vector<double> point_dig =  dig[i];
		double gz = point_dig[2];
		dig_depth += gz;
	}
	if (dig_depth > max_dig_depth) {
		return;
	}

	// копаем в выбранных точках  и добываем все попадающееся золото
	int gold_count = 0;
	for (unsigned int i = 0; i < dig.size(); i++) {
		std::vector<double> point_dig = dig[i];
		double gx = point_dig[0];
		double gy = point_dig[1];
		double gz = point_dig[2];
		for (unsigned j = 0; j < gold_can_dig.size(); j++) {
			std::vector<double> gold =  gold_can_dig[j];
			double gdx = gold[0];
			double gdy = gold[1];
			double gdz = gold[2];
			double gdg = gold[3];
			if (point_in_circle(gdx, gdy, gx, gy, DIG_RADIUS) &&
				(gdz <= gz)) {
				gold_count += gdg;
			}

		}
	}

	// обновляем максимум добытого золота
	if (gold_count > max_gold) {
		max_gold = gold_count;
	}
}

// генерируем сочетания позиций, в которых будем копать
void generate_combination(std::vector<std::vector<double> > & points_can_dig,
		std::vector<std::vector<double> > & gold_can_dig,
		int combination_count,
		int num, int last, std::vector<int> & combination, int max_dig_depth, int & max_gems) {
	if (num == combination_count) {
		for (int i = 0; i < combination_count; i++)
		//	std::cout << combination[i];
		//std::cout << std::endl;
		check_combination(points_can_dig, gold_can_dig, combination, max_dig_depth, max_gems);
	}
	for (unsigned i = last + 1; i <= points_can_dig.size(); i++) {
		combination.push_back(i);
		generate_combination(points_can_dig, gold_can_dig, combination_count, num + 1, i, combination, max_dig_depth, max_gems);
		combination.pop_back();
	}
}

int main() {
	std::ifstream infile("task6.in");

	int n, t, k, z;
	infile >> n >> t >> k >> z;

	std::vector<std::vector<double> > area;

	double x, y, max_x, max_y = 0;
    for (int point = 0; point < k; point++) {
    	infile >> x;
    	infile >> y;
    	std::vector<double> p;
    	p.push_back(x);
    	p.push_back(y);
    	area.push_back(p);
    	if (x > max_x) {
    		max_x = x;
    	}
    	if (y > max_y) {
    	    max_y = y;
    	}
    }

	double gold[n][4];

    for (int row = 0; row < n; row++) {
    	for (int col = 0; col < 4; col++) {
    		infile >> gold[row][col];
    	}
    }

    long long t_count = t * z;

    // поиск золота, которое можно копать
    std::vector<std::vector<double> > gold_can_dig;

    for (int row = 0; row < n; row++) {
    	double gx = gold[row][0];
    	double gy = gold[row][1];
    	double gz = gold[row][2];
    	double gg = gold[row][3];
    	if (point_in_polygon(gx, gy, area) && gz <= t_count) {
    		std::vector<double> g;
    		g.push_back(gx);
    		g.push_back(gy);
    		g.push_back(gz);
    		g.push_back(gg);
    		gold_can_dig.push_back(g);
    	}
    }

    // заполняем точки, в которых можно копать
    std::vector<std::vector<double> > points_can_dig;
    for (int x = 0; x < max_x; x++) {
    	for (int y = 0; y < max_y; y++) {
    		if (circle_in_polygon(area, x, y, DIG_RADIUS)) {
    			for (int g = 0; g < gold_can_dig.size(); g++) {
    				double gx = gold_can_dig[g][0];
    				double gy = gold_can_dig[g][1];
    				double gz = gold_can_dig[g][2];
    				if (point_in_circle(gx, gy, x, y, DIG_RADIUS)) {
    					// нашли золото в радиусе 0.5 м, рассматриваем вариант копки на глубину месторождения
    					std::vector<double> point;
    					point.push_back(x);
    					point.push_back(y);
    					point.push_back(gz);
    					points_can_dig.push_back(point);
    				}
    			}
    		}
    	}
    }

	int max_gold = 0;

	std::vector<int> combination;

	for (unsigned int i = 1; i <= points_can_dig.size(); i++) {
	  combination.clear();
	  generate_combination(points_can_dig, gold_can_dig, i, 0, 0, combination, t_count, max_gold);
	}

	std::ofstream outfile("task6.out");
    outfile << max_gold;

	return 0;
}
